#include "error.inc"

!vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv!
!                                                                      !
!  Module name: USR_DRAG                                               !
!                                                                      !
!  Purpose: Provide a hook for user defined drag law implementation.   !
!                                                                      !
!  This routine is called from inside fluid (TFM) and particle (DES)   !
!  loops. The fluid cell index (IJK) and phase (TFM) or particle index !
!  (DES) is passed.                                                    !
!                                                                      !
!  ***************************   WARNING   **************************  !
!  *----------------------------------------------------------------*  !
!  * The dummy arguments changed in the 2015-1 MFIX Release.        *  !
!  *                                                                *  !
!  *   1) Phase index (M) is now particle index (NP) for DES. This  *  !
!  *      is reflected in the name change M --> M_NP.               *  !
!  *                                                                *  !
!  *   2) The fluid velocity was added as a dummy argument. This    *  !
!  *      provides access to the interpolated gas velocity for      *  !
!  *      coupled DES simulations.                                  *  !
!  *                                                                *  !
!  ******************************************************************  !
!                                                                      !
!^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^!
! RADL


      SUBROUTINE DRAG_USR(IJK, M_NP, lDgA, EPg, Mug, ROg, VREL, DPM, &
         ROs, lUg, lVg, lWg)

      use error_manager
      USE param
      USE param1
      USE run, only : filter_size_ratio
      USE constant, only : GRAVITY
      USE geometry, only : VOL,AXY,DO_K
      USE discretelement, only: des_usr_var
      IMPLICIT NONE

! Index of fluid cell:
      INTEGER, INTENT(IN) :: IJK
! TFM SOLIDS --> Index of phase (M)
! DES SOLIDS --> Index of particle (NP); M = PIJK(NP,5)
      INTEGER, INTENT(IN) :: M_NP

! drag coefficient
      DOUBLE PRECISION, INTENT(OUT) :: lDgA
! gas volume fraction
      DOUBLE PRECISION, INTENT(IN) :: EPg
! gas laminar viscosity
      DOUBLE PRECISION, INTENT(IN) :: Mug
! gas density
      DOUBLE PRECISION, INTENT(IN) :: ROg
! Magnitude of gas-solids relative velocity
      DOUBLE PRECISION, INTENT(IN) :: VREL
! particle diameter of solids phase M or
! average particle diameter if PCF
      DOUBLE PRECISION, INTENT(IN) :: DPM
! particle density of solids phase M
      DOUBLE PRECISION, INTENT(IN) :: ROs
! fluid velocity components:
! o TFM: Averaged from faces to cell center
! o DES: Interpolated to the particle's position
      DOUBLE PRECISION, INTENT(IN) :: lUg, lVg, lWg

!^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
! Reynolds number
      DOUBLE PRECISION :: RE
! Single sphere drag coefficient
      DOUBLE PRECISION :: C_d
! terminal velocity
      DOUBLE PRECISION :: vt
! filter size
      DOUBLE PRECISION :: filtersize
      DOUBLE PRECISION :: Froude,L_char
! total solids volume fraction
      DOUBLE PRECISION :: EPs,phis
      DOUBLE PRECISION :: h_d, h_phip, f_filter,a_phip
! correction Parameter
      DOUBLE PRECISION :: Ccorr

      IF(Mug > ZERO) THEN
! Note the presence of gas volume fraction in ROPG
         RE = DPM*VREL*ROg/Mug
      ELSE
         RE = LARGE_NUMBER
      ENDIF

      IF (RE == ZERO) THEN
         lDgA = ZERO
         RETURN
      ENDIF

      IF(RE <= 1000.0D0)THEN
         C_d = (24.D0/(RE+SMALL_NUMBER)) * (ONE + 0.15D0*RE**0.687D0)
      ELSE
         C_d = 0.44D0
      ENDIF

      lDgA = 0.75D0 * C_d * VREL * ROg * EPg**(-2.65D0) / DPM

      IF(DO_K) THEN
         filtersize = 1.0 * (VOL(IJK)**(ONE/3.0d0))
      ELSE
         filtersize = 1.0 * DSQRT(AXY(IJK))
      ENDIF
! initialize
      h_d = ONE
! particle terminal settling velocity
      vt = GRAVITY*DPM*DPM*(ROs - ROg) / (18.0d0*Mug)

      Froude= vt**2/DPM/GRAVITY
      L_char= vt**2/GRAVITY*Froude**(-2.0d0/3.0d0)

      EPs = ONE - EPg
      phis = ONE -EPg

      IF (EPs .LT. 0.03d0) THEN
         h_phip = 0 + 7.97d0*(EPs-0) + 0*(EPs-0)**2d0 + 0*(EPs-0)**3d0
      ELSEIF (EPs .LT. 0.08d0) THEN
         h_phip = 0.239d0+4.64d0*(EPs-0.03d0)-4.41d0*(EPs-0.03d0)**2d0+253.63d0*(EPs-0.03d0)**3d0
      ELSEIF (EPs .LT. 0.12d0) THEN
         h_phip = 0.492d0 + 6.10d0*(EPs-0.08d0)+33.63d0*(EPs-0.08d0)**2d0-789.6d0*(EPs-0.08d0)**3d0
      ELSEIF (EPs .LT. 0.18d0) THEN
         h_phip = 0.739d0 +5.01d0*(EPs-0.12d0)-61.1d0*(EPs-0.12d0)**2d0+ 310.8d0*(EPs-0.12d0)**3d0
      ELSEIF (EPs .LE. 0.34d0) THEN
         h_phip = 0.887d0+1.03d0*(EPs-0.18d0)-5.17d0*(EPs-0.18d0)**2d0+5.99d0*(EPs-0.18)**3d0
      ELSEIF (EPs .LE. 0.48d0) THEN
         h_phip = 0.943d0-0.170d0*(EPs-0.34d0)-2.29d0*(EPs-0.34d0)**2d0-9.12d0*(EPs-0.34d0)**3d0
      ELSEIF (EPs .LE. 0.55d0) THEN
         h_phip = 0.850d0-1.35d0*(EPs-0.48d0)-6.13d0*(EPs-0.48d0)**2d0-132.6d0*(EPs-0.48d0)**3d0
      ELSEIF (EPs .LE. 0.60d0) THEN
         h_phip = 0.680d0 -2.34d0*(EPs-0.55d0)-225.2d0*(EPs-0.55d0)**2d0 + 0*(EPs-0.55d0)**3d0
      ELSE
         h_phip=ZERO
      ENDIF

      IF (EPs .LT. 0.016d0) THEN
         a_phip = 21.51d0 + 0*(EPs-0) + 0*(EPs-0)**2d0 + 0*(EPs-0)**3d0
      ELSEIF (EPs .LT. 0.10d0) THEN
         a_phip = 1.96d0+29.40d0*(EPs-0) +164.91d0*(EPs-0)**2d0 -1923d0*(EPs-0)**3d0
      ELSEIF (EPs .LT. 0.18d0) THEN
         a_phip = 4.63d0 + 4.68d0*(EPs-0.1d0) -412.04d0*(EPs-0.1d0)**2d0+2254d0*(EPs-0.1d0)**3d0
      ELSEIF (EPs .LT. 0.25d0) THEN
         a_phip = 3.52d0 -17.99d0*(EPs-0.18d0) +128.80d0*(EPs-0.18d0)**2d0-603d0*(EPs-0.18d0)**3d0
      ELSEIF (EPs .LE. 0.40d0) THEN
         a_phip = 2.68d0 -8.82d0*(EPs-0.25d0) +2.18d0*(EPs-0.25d0)**2d0 + 112.33d0*(EPs-0.25d0)**3d0
      ELSE
         a_phip = 1.79d0 + 0*(EPs-0) + 0*(EPs-0)**2d0 + 0*(EPs-0)**3d0
      ENDIF

      f_filter = ONE/(a_phip*L_char/filtersize+1.0d0)
      h_d = ONE-f_filter*h_phip

! Ccorr = exp(-k(l-1)), l equals CG-Factor
      !Ccorr = exp(-0.05*(200-1))
      Ccorr = 0.8

!Filtered Drag
      lDgA = lDgA*h_d*Ccorr     
      !des_usr_var(1, M_NP) = h_d*Ccorr 


      RETURN
      END SUBROUTINE DRAG_USR
